module BagTest where

import BagWithHeap
import Test.HUnit


-- Lokale Listen und Bags
fullList :: [Int]
fullList  = [1,2,3]

fullList2 :: [Int]
fullList2 = [3,4,5]

fullBag = mkBag fullList
fullBag2 = mkBag fullList2

emptyList :: [Int]
emptyList = []


-- Testcases, abgeleitet aus den Bag-Axiomen


-- Option 1, mkBag

-- isEmpty (mkBag xs)              = null xs
test1a = TestCase (assertEqual "Axiom1a" (isEmpty (mkBag emptyList)) (null emptyList))
test1b = TestCase (assertEqual "Axiom1b" (isEmpty (mkBag fullList)) (null fullList))

-- union (mkBag xs) (mkBag ys)     = mkBag(xs ++ ys)
test2 = TestCase (assertEqual "Axiom2" (union (mkBag fullList) (mkBag fullList2)) (mkBag(fullList ++ fullList2)))

-- minBag (mkBag xs)               = minlist xs
test3 = TestCase (assertEqual "Axiom3" (minBag (mkBag fullList)) (head fullList)) 

-- delMin(mkBag xs)                = mkBag (deleteMin xs)
test4 = TestCase (assertEqual "Axiom4" (delMin(mkBag fullList)) (mkBag (tail fullList)))


-- Option 2, insert empty

-- insert x (insert y xb)          = insert y (insert x xb)
test5 = TestCase (assertEqual "Axiom5" (insert 1 (insert 2 empty )) (insert 2 (insert 1 empty)))

-- mkBag                           = foldr insert empty
test6 = TestCase (assertEqual "Axiom6" (mkBag fullList) (foldr insert empty fullList))

-- union xb empty                  = xb
test7 = TestCase (assertEqual "Axiom7" (union fullBag empty) (fullBag))

-- union xb (insert y yb)          = insert y (union xb yb)
test8 = TestCase (assertEqual "Axiom8" (union fullBag (insert 7 fullBag2)) (insert 7 (union fullBag fullBag2)))

-- minBag (insert x empty)         = x
test9 = TestCase (assertEqual "Axiom9" (minBag (insert 1 empty)) (1))

-- minBag (insert x (insert y xb)) = x `min` minBag (insert y xb)
test10 = TestCase (assertEqual "Axiom10" (minBag (insert 0 (insert 7 fullBag))) (0 `min` minBag( insert 7 fullBag)))

-- delMin (insert x empty)         = empty
test11 = TestCase (assertEqual "Axiom11" (delMin (insert 1 empty)) (empty))

-- insert (minBag xb) (delMin xb)  = xb
test12 = TestCase (assertEqual "Axiom12" (insert (minBag fullBag) (delMin fullBag)) (fullBag))



tests = TestList [TestLabel "test1a"  test1a,
                  TestLabel "test1b"  test1b,
                  TestLabel "test2"   test2,
                  TestLabel "test3"   test3,
                  TestLabel "test4"   test4,
                  TestLabel "test5"   test5,
                  TestLabel "test6"   test6,
                  TestLabel "test7"   test7,
                  TestLabel "test8"   test8,
                  TestLabel "test9"   test9,
                  TestLabel "test10"  test10,
                  TestLabel "test11"  test11,
                  TestLabel "test12"  test12]

runTests = runTestTT tests
